"""
Load hourly table from detail table
"""
#
# usage:
#    python t4_agg_db_v3.py [options] 
#
#

import sys, MySQLdb
from optparse import OptionParser
try:
    import rdb
    import types
    class Cursor:
        def __init__(self):
            self._curs = None
        def _check_executed(self):
            if not self._curs:
                raise "execute() first"
        def execute(self, stmt, args = None):
            """Execute a query.

            query -- string, query to execute on server
            args -- optional sequence or mapping, parameters to use with query.
            """
            stmt = stmt.replace('%s','?')
            self._curs = rdb.statement(stmt)
            if args is None:
                self._curs.execute()
            elif type(args) in (types.TupleType, types.ListType):
                self._curs.execute(*args)
            else:
                self._curs.execute(args)
            return None
        def fetchall(self):
            """Fetchs all available rows from the cursor."""
            self._check_executed()
            return self._curs.fetchall()
except:
    pass

Backend = None

def get_tables_name(curs):
    "Return all tables name"
    req = "select distinct table_name from repository"
    curs.execute(req)
    res = curs.fetchall()
    return [n[0].strip() for n in res]

def get_columns_name(table_name, curs):
    "Return all columns name for a specified table"
    req = "select distinct column_name from repository where table_name = %s"
    curs.execute(req, table_name)
    res = curs.fetchall()
    return [n[0].strip() for n in res]


def get_node_id(node_name, curs):
    "Convert a node name to a node id"
    req = "select node_id from node where node_name = %s"
    curs.execute(req, node_name)
    return curs.fetchall()

def build_request(agg_schema, tbl_name, ts_from, ts_to,
                  agg_level, curs):
    req = "insert into %s.%s (select node_id," % (agg_schema, tbl_name)
    req += "cast(cast(SAMPLE_TIME as char(13)) || ':00:00' as timestamp(0)) as ts,"
    lst = ["avg(%s)" % cn for cn in get_columns_name(tbl_name, curs)]
    req += ",".join(lst)
    req += " from %s " % (tbl_name,)
    if ts_from or ts_to:
        req += "where "
        if ts_from:
            req += "SAMPLE_TIME >= timestamp'%s 00:00:00' " % ts_from
        if ts_from and ts_to:
            req += "and "
        if ts_to:
            req += "SAMPLE_TIME < timestamp'%s 00:00:00' " % ts_to
    req += """ group by node_id,
cast(cast(SAMPLE_TIME as char(13)) || ':00:00' as timestamp(0))
order by node_id,ts)"""
    return req

def main():
    "Main routine"
    global Backend
    usage = "usage: python %prog [options]"
    parser = OptionParser(usage = usage)
    parser.add_option("-b", "--backend", action="store", type="choice",
                      choices=["rdb", "mysql"], default="rdb",
                      help="Choice storage backend")
    parser.add_option("-f", "--from", action="store", type="string",
                      dest="ts_from", default=None,
                      help="starting date to aggregate ")
    parser.add_option("-t", "--to", action="store", type="string",
                      dest="ts_to", default=None,
                      help="ending date to aggregate")
    parser.add_option("-a", "--aggregate", action="store", type="choice",
                      choices=["10", "60"], dest="agg_level", default="60",
                      help="Aggregate level (currently 10 or 60)")
    # mysql specific
    parser.add_option("-n", "--node", action="store", type="string",
                      dest="host", default="localhost",
                      help="MySQL, connect to host")
    parser.add_option("-u", "--user", action="store", type="string",
                      dest="user", default="",
                      help="MySQL, user for login if not current user")
    parser.add_option("-p", "--password", action="store", type="string",
                      dest="password", default="",
                      help="MySQL, password to use when connecting to server")
    parser.add_option("-d", "--database", action="store", type="string",
                      dest="schema", help="name of mysql database to used")
    # rdb specific
    parser.add_option("-r", "--rdb_database", action="store", type="string",
                      dest="rdb_db", default="T4_RDB",
                      help="Rdb, database used")
    parser.add_option("-s", "--schema", action="store", type="string",
                      dest="schema", help="name of rdb schema to used")
    (options, args) = parser.parse_args()

    Backend = options.backend.lower()

    if options.agg_level == "10":
        agg_schema = options.schema.lower() + '_tm'
    else:
        agg_schema = options.schema.lower() + '_h'
    detail_schema = options.schema

    if options.backend == 'mysql':
        cnx = MySQLdb.connect(host=host, db=schema, user=user, passwd=passwd)
        curs = cnx.cursor()
        curs.execute('set autocommit = 0')
        curs.execute("SET SESSION sql_mode='ansi'")
        raise "mysql currently not supported"
    elif options.backend == 'rdb':
        rdb.statement("attach 'filename %s'" % (options.rdb_db,)).execute()
        rdb.statement("set schema '%s'" % (detail_schema,)).execute()
        rdb.statement("set transaction read write").execute()
        curs = Cursor()

    for tbln in get_tables_name(curs):
        req = build_request(agg_schema, tbln,
                            options.ts_from, options.ts_to,
                            options.agg_level, curs)
        print req
        curs.execute(req)
        print

    curs.execute('commit')

if __name__ == '__main__':
    main()
