#
# python t4_loader_v3.py [options] schema filename
#    where
#         filename: file containing T4 data
#
#

import sys, csv, MySQLdb
from optparse import OptionParser
# import pprint
try:
    import rdb
    import types
    class Cursor:
        def __init__(self):
            self._curs = None
        def _check_executed(self):
            if not self._curs:
                raise "execute() first"
        def execute(self, stmt, args = None):
            """Execute a query.

            query -- string, query to execute on server
            args -- optional sequence or mapping, parameters to use with query.
            """
            stmt = stmt.replace('%s','?')
            self._curs = rdb.statement(stmt)
            if args is None:
                self._curs.execute()
            elif type(args) in (types.TupleType, types.ListType):
                self._curs.execute(*args)
            else:
                self._curs.execute(args)
            return None
        def fetchall(self):
            """Fetchs all available rows from the cursor."""
            self._check_executed()
            return self._curs.fetchall()
except:
    pass

Backend = None

def date_vms_to_ansi(date):
    "Convert a date VMS format to a date ANSI format"
    lst = ['JAN', 'FEB', 'MAR', 'APR', 'MAY' , 'JUN', 'JUL', 'AUG',
           'SEP', 'OCT', 'NOV', 'DEC']
    fld = date.split('-')
    date = "%04d-%02d-%02d" % (int(fld[2]), lst.index(fld[1].upper()) + 1,
                               int(fld[0]))
    return date

def datetime_vms_to_ansi(timestamp):
    "Convert a timestamp VMS format to a timestamp ANSI format"
    date = timestamp[:11]
    time = timestamp[12:].strip()
    return '%s %s' % (date_vms_to_ansi(date), time)

def get_node_id(node_name, curs = None):
    "Convert a node name to a node id"
    req = "select node_id from node where node_name = %s"
    curs.execute(req, node_name)
    return curs.fetchall()
        
def add_node(node_name, curs = None):
    """
    Insert a new line into the node table
    """
    req = "insert into node (node_name) values (%s)"
    curs.execute(req, node_name)
    return get_node_id(node_name, curs)

        
def sql_exec(req, values, curs = None):
    "Execute a SQL statement"
#    pprint.pprint(req)
#    pprint.pprint(values)
    curs.execute(req, values)

def normalize_table_name(table_name):
    "Remove ':' and replace '.' by '_' into table_name"
    return table_name.replace('.', '_').replace(':', '')

def extract_names(name):
    """
    return
            - None if Sample Time column
        or
            - table name, column name
    """
    name = name.lstrip()
    fclass, fname = name.split(']')
    if fname == 'Sample Time': # new table
        return None
    else:
        return normalize_table_name(fclass.replace('[','')), name

class Table:
    """
    Table class
    """
    def __init__(self, name, curs = None, columns_name = None):
        if columns_name:
            self.columns_name = columns_name
        else:
            self.columns_name = []
        self.node_id = None
        self.timestamp = None
        self._sql_columns_name = None
        self.name = name
        self.value = []
        self._req = None
        self.curs = curs
        self.repository = {}
        rep = self._repository()
        for (col, item) in rep:
            self.repository[item.rstrip()] = col.rstrip()
    def clear_values(self):
        self.value = []
    def add_column_name(self, column_name):
        self.columns_name.append(column_name)
    def add_value(self, val):
        self.value.append(float(val))
    def _repository(self):
        req = "select column_name, item_name from repository where table_name = ?"
        curs = self.curs
        curs.execute(req, self.name)
        return curs.fetchall()
    def sql_columns_name(self):
        "build the columns_name if necessary return it, else just return it"
        if not self._sql_columns_name:
            col_names = ['"%s"' % (self.repository[item],)
                         for item in self.columns_name]
            col_names = ['node_id', 'sample_time'] + col_names
            col_names = ','.join(col_names)
            if Backend == 'mysql':
                col_names = col_names.replace('%', '%%')
            self._sql_columns_name = col_names
            return self._sql_columns_name
    def sql_insert(self, curs = None):
        "Build and Execute the SQL insert statement"
        if not self._req:
            self._req = "insert into %s (%s) values " \
                        % (self.name, self.sql_columns_name()) + \
                        '(' + '%s,' * (len(self.columns_name) + 1) + \
                        '%s' + ')'
        try:
            sql_exec(self._req, [self.node_id, self.timestamp] + self.value,
                     curs)
        except:
            print self._req, self.value
            raise

        self.clear_values()

def main():
    "Main routine"
    global Backend
    usage = "usage: python %prog [options] input_file"
    parser = OptionParser(usage = usage)
    parser.add_option("-b", "--backend", action="store", type="choice",
                      choices=["rdb", "mysql"], default="mysql",
                      help="Choice storage backend")
    # mysql specific
    parser.add_option("-n", "--node", action="store", type="string",
                      dest="host", default="localhost",
                      help="MySQL, connect to host")
    parser.add_option("-u", "--user", action="store", type="string",
                      dest="user", default="",
                      help="MySQL, user for login if not current user")
    parser.add_option("-p", "--password", action="store", type="string",
                      dest="password", default="",
                      help="MySQL, password to use when connecting to server")
    parser.add_option("-d", "--database", action="store", type="string",
                      dest="schema", help="name of mysql database to used")
    # rdb specific
    parser.add_option("-r", "--rdb_database", action="store", type="string",
                      dest="rdb_db", default="SQL$DATABASE",
                      help="Rdb, database used")
    parser.add_option("-s", "--schema", action="store", type="string",
                      dest="schema", help="name of rdb schema to used")
    (options, args) = parser.parse_args()
    parser = OptionParser(usage = usage)
    host = options.host
    user = options.user
    passwd = options.password
    schema = options.schema.upper()
    filename = args[0]

    Backend = options.backend.lower()

    if options.backend == 'mysql':
        cnx = MySQLdb.connect(host=host, db=schema, user=user, passwd=passwd)
        curs = cnx.cursor()
        curs.execute('set autocommit = 0')
        curs.execute("SET SESSION sql_mode='ansi'")
    elif options.backend == 'rdb':
        rdb.statement("attach 'filename %s'" % (options.rdb_db,)).execute()
        rdb.statement("set schema '%s'" % (schema,)).execute()
        rdb.statement("set transaction read write").execute()
        curs = Cursor()

    reader = csv.reader(file(filename))

    row = reader.next()
    node = row[0]
    row = reader.next()
    date = row[0]

    date = date_vms_to_ansi(date)

    row = reader.next()

    print 'Node: %s Date: %s' % (node, date)

    node_id = get_node_id(node, curs)
    if not node_id:
        node_id = add_node(node, curs)

    node_id = node_id[0][0]
    header = row = reader.next()
    col_tbl = []

    tables = {}
    for i in range(len(header)):
        info = extract_names(header[i])
        if info == None:
            col_tbl.append(None)
            continue
        else:
            if not tables.has_key(info[0].lower()):
                tables[info[0].lower()] = Table(info[0].lower(), curs)
            col_tbl.append(tables[info[0].lower()])
            tables[info[0].lower()].add_column_name('%s'% info[1].lstrip())

#    for v in tables:
#        print 'table', v
#        pprint.pprint(tables[v].columns_name)

    for row in reader:
        timestamp = datetime_vms_to_ansi(row[0])
        for tbl in tables.itervalues():
            tbl.node_id = node_id
            tbl.timestamp = timestamp
        i = 0
        for col in row:
            if col_tbl[i]:
                col_tbl[i].add_value(col)
            i += 1
        for tbl in tables.itervalues():
            tbl.sql_insert(curs)

    curs.execute('commit')

if __name__ == '__main__':
    main()
