#
# python t4_purge_v3.py [options] until [node_name]
#
#      until: purge data until (date ansi)
#      node_name: mode to purge
#
#

import sys, csv, MySQLdb
from optparse import OptionParser
# import pprint
try:
    import rdb
    import types
    class Cursor:
        def __init__(self):
            self._curs = None
        def _check_executed(self):
            if not self._curs:
                raise "execute() first"
        def execute(self, stmt, args = None):
            """Execute a query.

            query -- string, query to execute on server
            args -- optional sequence or mapping, parameters to use with query.
            """
            stmt = stmt.replace('%s','?')
            self._curs = rdb.statement(stmt)
            if args is None:
                self._curs.execute()
            elif type(args) in (types.TupleType, types.ListType):
                self._curs.execute(*args)
            else:
                self._curs.execute(args)
            return None
        def fetchall(self):
            """Fetchs all available rows from the cursor."""
            self._check_executed()
            return self._curs.fetchall()
except:
    pass

Backend = None

def get_tables_name(curs):
    "Return all tables name"
    req = "select distinct table_name from repository"
    curs.execute(req)
    res = curs.fetchall()
    return [n[0].strip() for n in res]

def get_node_id(node_name, curs):
    "Convert a node name to a node id"
    req = "select node_id from node where node_name = %s"
    curs.execute(req, node_name)
    return curs.fetchall()
        
def sql_exec(req, values, curs = None):
    "Execute a SQL statement"
#    pprint.pprint(req)
#    pprint.pprint(values)
    curs.execute(req, values)

def main():
    "Main routine"
    global Backend
    usage = "usage: python %prog [options] date_ansi [node_to_purge]"
    parser = OptionParser(usage = usage)
    parser.add_option("-b", "--backend", action="store", type="choice",
                      choices=["rdb", "mysql"], default="mysql",
                      help="Choice storage backend")
    parser.add_option("-a", "--aggregate", action="store", type="choice",
                      choices=["10", "60"], dest="agg_level", default=None,
                      help="Aggregate level (currently 10 or 60)")
    # mysql specific
    parser.add_option("-n", "--node", action="store", type="string",
                      dest="host", default="localhost",
                      help="MySQL, connect to host")
    parser.add_option("-u", "--user", action="store", type="string",
                      dest="user", default="",
                      help="MySQL, user for login if not current user")
    parser.add_option("-p", "--password", action="store", type="string",
                      dest="password", default="",
                      help="MySQL, password to use when connecting to server")
    parser.add_option("-d", "--database", action="store", type="string",
                      dest="schema", help="name of mysql database to used")
    # rdb specific
    parser.add_option("-r", "--rdb_database", action="store", type="string",
                      dest="rdb_db", default="T4_RDB",
                      help="Rdb, database used")
    parser.add_option("-s", "--schema", action="store", type="string",
                      dest="schema", help="name of rdb schema to used")
    (options, args) = parser.parse_args()
    parser = OptionParser(usage = usage)
    host = options.host
    user = options.user
    passwd = options.password
    schema = options.schema.upper()
    purged_schema = schema
    if options.agg_level == "60":
        purged_schema += '_h'
    elif options.agg_level == "10":
        purged_schema += '_tm'

    until = args[0]
    if len(args) > 1:
        node_name = args[1].upper()
    else:
        node_name = None

    Backend = options.backend.lower()

    if options.backend == 'mysql':
        cnx = MySQLdb.connect(host=host, db=schema, user=user, passwd=passwd)
        curs = cnx.cursor()
        curs.execute('set autocommit = 0')
        curs.execute("SET SESSION sql_mode='ansi'")
    elif options.backend == 'rdb':
        rdb.statement("attach 'filename %s'" % (options.rdb_db,)).execute()
        rdb.statement("set schema '%s'" % (schema,)).execute()
        rdb.statement("set transaction read write").execute()
        curs = Cursor()

    if node_name:
        node_id = get_node_id(node_name, curs)[0]
    else:
        node_id = None

    tables = get_tables_name(curs)

    for tbl in tables:
        req = "delete from %s.%s where SAMPLE_TIME <= " % (purged_schema, tbl)
        if Backend == 'mysql':
            req += " '%s'" % until
        else:
            req += " date ansi'%s'" % until
        if node_id is not None:
            req += " and node_id = %d" % node_id
        print req
        curs.execute(req)
    curs.execute('commit')

if __name__ == '__main__':
    main()
