"""
Create table from a MySQL or Rdb database from a T4 detail schema/database
"""
#
# usage:
#    python t4_tables_agg_v3.py [options] output_file
#    where
#         output_file: file receiving SQL statements
#
#
import sys, MySQLdb
from optparse import OptionParser
# import pprint

try:
    import rdb
    import types
    class Cursor:
        def __init__(self):
            self._curs = None
        def _check_executed(self):
            if not self._curs:
                raise "execute() first"
        def execute(self, stmt, args = None):
            """Execute a query.

            query -- string, query to execute on server
            args -- optional sequence or mapping, parameters to use with query.
            """
            stmt = stmt.replace('%s','?')
            self._curs = rdb.statement(stmt)
            if args is None:
                self._curs.execute()
            elif type(args) in (types.TupleType, types.ListType):
                self._curs.execute(*args)
            else:
                self._curs.execute(args)
            return None
        def fetchall(self):
            """Fetchs all available rows from the cursor."""
            self._check_executed()
            return self._curs.fetchall()
except:
    pass

Backend = None

def get_tables_name(curs):
    "Return all tables name"
    req = "select distinct table_name from repository"
    curs.execute(req)
    res = curs.fetchall()
    return [n[0].strip() for n in res]

def get_item_infos(tbln, curs):
    "Return all info for a specified table_name from repository"
    req = "select item_id, column_name, item_name "
    req += "from repository where table_name = %s order by item_id"
    curs.execute(req, tbln)
    res = curs.fetchall()
    rep = [(v[0], v[1].rstrip(), v[2].rstrip()) for v in res]
    return rep

def sql_exec(req, values, curs = None):
    "Execute a SQL statement"
#    pprint.pprint(req)
#    pprint.pprint(values)
    curs.execute(req, values)

def gen_security(fres, options, name):
    if options.writer:
        print >> fres, 'grant select, insert, update, delete'
        print >> fres, '    on table "%s" to %s position 2;' % \
              (name, options.writer)
        posread = 3
    else:
        posread = 2
    if options.reader:
        print >> fres, 'grant select'
        print >> fres, '    on table "%s" to %s position %d;' % \
              (name, options.reader, posread)

class Table:
    """
    Table class
    """
    def __init__(self, name, options, curs, backend):
        self.backend = backend
        self.name = name
        self.columns_name = []
        for info in get_item_infos(name, curs):
            self.add_column(info)
    def add_column(self, colinf):
        # rdb maximum column name size is 31 characters
        item_id = colinf[0]
        column_name = colinf[1].rstrip()
        item_name = colinf[2].rstrip()
        self.columns_name.append(column_name)
    def generate_create(self, fres, options):
        print >> fres
        if options.with_drop_table:
            if options.backend == 'mysql':
                print >> fres, 'drop table if exists %s;' % self.name
            else:
                print >> fres, 'drop table "%s";' % self.name.upper()
        if options.backend == 'rdb':
            print >> fres, 'create table "%s" (' % self.name.upper()
            print >> fres, '    node_id integer NOT NULL not deferrable,'
        else:
            print >> fres, 'create table %s (' % self.name
            print >> fres, '    node_id integer NOT NULL,'
        print >> fres, '    sample_time timestamp(0),'
        for colname in self.columns_name:
            print >> fres, '    "%s" float,' % colname
        print >> fres, '    PRIMARY KEY (node_id, sample_time)',
        if options.backend == 'mysql':
            print >> fres
            print >> fres, ') type=InnoDB CHECKSUM=1 ROW_FORMAT=COMPRESSED;'
        elif options.backend == 'rdb':
            print >> fres, ' not deferrable'
            print >> fres, ');'
            gen_security(fres, options, self.name.upper())
            print >> fres, 'create unique index %s_idx1 on "%s" (' %\
                  (self.name, self.name.upper())
            print >> fres, '    node_id, sample_time);'

def main():
    "Main routine"
    global Backend
    usage = "usage: python %prog [options] output_file"
    parser = OptionParser(usage = usage)
    parser.add_option("-d", "--drop_table", action="store_true",
                      dest="with_drop_table", default=False,
                      help="Generate drop table before create table")
    parser.add_option("-a", "--aggregate", action="store", type="choice",
                      choices=["10", "60"], dest="agg_level", default="60",
                      help="Aggregate level (currently 10 or 60)")
    parser.add_option("-r", "--rdb_database", action="store", type="string",
                      dest="rdb_db", default="SQL$DATABASE",
                      help="Rdb, database used")
    parser.add_option("-b", "--backend", action="store", type="choice",
                      choices=["rdb", "mysql"], default="mysql",
                      help="Choice storage backend, rdb or mysql")
    parser.add_option("-s", "--schema", action="store", type="string",
                      dest="schema", default="t4",
                      help="name of rdb schema or mysql database to create")
    parser.add_option("-W", "--writer", action="store",type="string", 
                      dest="writer", default=None)
    parser.add_option("-R", "--reader", action="store",type="string", 
                      dest="reader", default=None)
    (options, args) = parser.parse_args()

    schema = options.schema.lower()
    if options.agg_level == "10":
        schema_agg = options.schema.lower() + '_tm'
    else:
        schema_agg = options.schema.lower() + '_h'

    Backend = options.backend.lower()

    output_file = args[0]

    fres = open(output_file, 'w')

    if options.backend == 'mysql':
        cnx = MySQLdb.connect(host=host, db=schema, user=user, passwd=passwd)
        curs = cnx.cursor()
        curs.execute('set autocommit = 0')
        curs.execute("SET SESSION sql_mode='ansi'")
    elif options.backend == 'rdb':
        rdb.statement("attach 'filename %s'" % (options.rdb_db,)).execute()
        rdb.statement("set schema '%s'" % (options.schema,)).execute()
        rdb.statement("set transaction read write").execute()
        curs = Cursor()

    if options.backend == 'mysql':
        print >> fres, "SET SESSION sql_mode='ansi';"
        print >> fres, "create database %s;" % (schema,)
        print >> fres, "use %s;" % (schema_agg,)
    elif options.backend == 'rdb':
        print >> fres, "-- create database filename db multischema is on;"
        print >> fres, "create schema %s;" % (schema_agg,)
        print >> fres, "set schema '%s';" % (schema_agg,)
        print >> fres, "set quoting rules 'SQL92';"
    print >> fres

    tables = {}
    for tbln in get_tables_name(curs):
        tables[tbln] = Table(tbln, options, curs, options.backend)

    for table in tables.itervalues():
        table.generate_create(fres, options)

    if options.backend == 'rdb':
        print >> fres, "-- commit work;"
    fres.close()

if __name__ == '__main__':
    main()
